#include "headers.h"

MODULE = Arisa::Network  PACKAGE = Arisa
PROTOTYPES: ENABLE

Arisa::Network
network_find(name = NULL)
	char *name
CODE:
	RETVAL = irc_network_find(name);
OUTPUT:
	RETVAL

void
networks()
PREINIT:
	int i;
PPCODE:
	LOCK(global);
	for(i = 0; i < global->no_networks; ++i) {
		network_t *n = global->networks[i];
		LOCK(n);
		if(n->deleted == 0)
			XPUSHs(sv_2mortal(
				create_ptr_object("Arisa::Network",n)));
		UNLOCK(n);
	}
	UNLOCK(global);

Arisa::Network
network_add(name = NULL, nick = NULL, server = NULL)
	char *name
	char *nick
	char *server
CODE:
	RETVAL = NULL;
	if(name != NULL) {
		network_t *n;
		
		LOCK(global);
		n = alloc_network(global->network_settings);
		UNLOCK(global);
		
		n->name = xstrdup(name);
		if(nick != NULL) {
			if(n->settings->nick != NULL)
				xfree(n->settings->nick);
			n->settings->nick = xstrdup(nick);
		}
		if(server != NULL)
			PTRARR_ADD(&(n->servers),&(n->no_servers),xstrdup(server));
		if(irc_network_add(n) == 0)
			RETVAL = n;
		else
			free_network(n);
	}
OUTPUT:
	RETVAL

	
#*******************************
MODULE = Arisa::Network  PACKAGE = Arisa::Network
#*******************************

SV*
delete(network)
	Arisa::Network network
CODE:
	RETVAL = &PL_sv_no;
	if(irc_network_valid(network)) {
		if(irc_network_del(network) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

SV*
name(network, new_name = NULL)
	Arisa::Network network
	char *new_name
CODE:
	RETVAL = &PL_sv_no;
	if(irc_network_valid(network)) {
		if(new_name != NULL) {
			if(irc_network_rename(network,new_name) == 0)
				RETVAL = &PL_sv_yes;
		} else {
			LOCK(network);
			RETVAL = new_pv(network->name);
			UNLOCK(network);
		}
	}
OUTPUT:
	RETVAL

void
info(n)
	Arisa::Network n
PREINIT:
	HV *hv = NULL;
PPCODE:
	if(irc_network_valid(n)) {
		hv = newHV();
		LOCK(n);
		hv_store(hv,"state",5,new_pv(networkstate_to_str(n->state)),0);
		if(n->info != NULL && n->state != NETWORK_OFFLINE) {
			LOCK(n->info);
			hv_store(hv,"nick",4,
				new_pv(n->info->nick),0);
			hv_store(hv,"server_addr",11,
				new_pv(n->info->server),0);
			hv_store(hv,"server_port",11,
				newSViv(n->info->port),0);
			if(n->state == NETWORK_CONNECTED) {
				hv_store(hv,"connected",9,
					newSVuv(n->info->connected),0);
			}
			hv_store(hv,"p_squeue",8,
				newSViv(pqueue_length(&n->info->p_squeue)),0);
			hv_store(hv,"j_squeue",8,
				newSViv(pqueue_length(&n->info->j_squeue)),0);
			hv_store(hv,"d_squeue",8,
				newSViv(pqueue_length(&n->info->d_squeue)),0);
			hv_store(hv,"r_squeue",8,
				newSViv(pqueue_length(&n->info->r_squeue)),0);
			UNLOCK(n->info);
		}	
		UNLOCK(n);
	}
	RET_REF(hv);

void
nick_info(n, nick = NULL)
	Arisa::Network n
	char *nick
PREINIT:
	HV *hv = NULL;
PPCODE:
	if(irc_network_valid(n) && nick != NULL) {
		LOCK(n);
		if(n->info != NULL) {
			chanmode_t *cms = NULL;
			char buffer[4];
			int i,ret;
			
			LOCK(n->info);
			ret = nickhash_query_nick(n->info->nick_tracker,
					nick,&cms);
			UNLOCK(n->info);

			if(ret > 0) {
				hv = newHV();
				for(i = 0; i < ret; ++i) {
					str_from_mode(buffer,sizeof(buffer),
							cms[i].mode);
					LOCK(cms[i].channel);
					hv_store(hv,
						cms[i].channel->name,
						strlen(cms[i].channel->name),
						new_pv(buffer),0);
					UNLOCK(cms[i].channel);
				}
			}
			if(cms != NULL)
				xfree(cms);
		}
		UNLOCK(n);
	}
	RET_REF(hv);
	
void
on_connects(network, ...)
	Arisa::Network network
PPCODE:
	if(irc_network_valid(network)) {
		LOCK(network);
		LOAD_RET_STRARR(network->on_connects,network->no_on_connects);
		UNLOCK(network);
	}
	
int
send_raw(network, msg = NULL)
	Arisa::Network network
	char *msg
CODE:
	if(msg != NULL && irc_network_valid(network))
		RETVAL = irc_send_raw(network,msg) == 0 ? 1 : 0;
	else
		RETVAL = 1;
OUTPUT:
	RETVAL

SV*
send_msg(network, target = NULL, msg = NULL)
	Arisa::Network network
	char *target
	char *msg
CODE:
	RETVAL = &PL_sv_no;
	if(target != NULL && msg != NULL && irc_network_valid(network)) {
		if(irc_send_msg(network,target,msg) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

SV*
send_notice(network, target = NULL, msg = NULL)
	Arisa::Network network
	char *target
	char *msg
CODE:
	RETVAL = &PL_sv_no;
	if(target != NULL && msg != NULL && irc_network_valid(network)) {
		if(irc_send_notice(network,target,msg) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

void
servers(network, ...)
	Arisa::Network network
PPCODE:
	if(irc_network_valid(network)) {
		LOCK(network);
		LOAD_RET_STRARR(network->servers,network->no_servers);
		UNLOCK(network);
	}
	
void
settings(network, ...)
	Arisa::Network network
PREINIT:
	HV *hv = NULL;
PPCODE:
	if(irc_network_valid(network)) {
		hv = newHV();
		perl_apply_settings(network_apply_setting,network);
		perl_read_settings(network_read_settings,network,hv);
	}
	RET_REF(hv);

SV*
start(network)
	Arisa::Network network
CODE:
	RETVAL = &PL_sv_no;
	if(irc_network_valid(network)) {
		LOCK(network);
		if(network->state == NETWORK_OFFLINE) {
			if(irc_network_run(network) != -1)
				RETVAL = &PL_sv_yes;
		}
		UNLOCK(network);
	}
OUTPUT:
	RETVAL

SV*
stop(network)
	Arisa::Network network
CODE:
	RETVAL = &PL_sv_no;
	if(irc_network_valid(network)) {
		LOCK(network);
		if(network->state != NETWORK_OFFLINE && network->info != NULL) {
			thread_t *t = irc_network_stop(network);
			UNLOCK(network);
			if(t != NULL) {
				thread_join(t,NULL);
				RETVAL = &PL_sv_yes;
			}
		} else
			UNLOCK(network);
	}
OUTPUT:
	RETVAL

void
taglines(network, ...)
	Arisa::Network network
PPCODE:
	if(irc_network_valid(network)) {
		LOCK(network);
		LOAD_RET_STRARR(network->settings->taglines,
			network->settings->no_taglines);
		UNLOCK(network);
	}
